'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
/**
 * A Slate plugin that auto wrap selected block with some custom block.
 *
 * @param {Object} opts
 * return {Object}
 */

function AutoWrapBlock(opts) {
  if (!opts.type) throw new Error('You must provide a `type` option.');
  if (!opts.condition) throw new Error('You must provide a `condition` option.');
  if (!opts.component) throw new Error('You must provide a `component` option.');

  /**
   * On select.
   *
   * @param  {Event} e
   * @param  {Object} data
   * @param  {State} state
   * @param  {Function} getState
   * @param  {Object} props
   */

  function onSelect(e, data, state, _ref) {
    var getState = _ref.getState;
    var props = _ref.props;

    setTimeout(function () {
      return autoWrap(getState, props.onChange);
    }, 1);
  }

  /**
   * On key down.
   *
   * @param  {Event} e
   * @param  {Object} data
   * @param  {State} state
   * @param  {Function} getState
   * @param  {Object} props
   */

  function onKeyDown(e, data, state, _ref2) {
    var getState = _ref2.getState;
    var props = _ref2.props;

    setTimeout(function () {
      return autoWrap(getState, props.onChange);
    }, 1);
  }

  /**
   * Get schema
   *
   * @param  {Object} node
   * @return {Function}
   */

  function getSchema(node) {
    var schema = {
      nodes: {}
    };
    schema.nodes['' + opts.type] = opts.component;
    return schema;
  }

  /**
   * Get wrapped block keys.
   *
   * @param  {List} nodes
   * @return {List}
   */

  function getWrappedBlockKeys(nodes) {
    return nodes.filter(function (node) {
      return node.type === opts.type;
    }).map(function (node) {
      return node.nodes.first().nodes.first().key;
    });
  }

  /**
   * Remove all wrapped blocks.
   *
   * @param  {State} wrappedBlockState
   * @return {State}
   */

  function removeAllWrappedBlock(wrappedBlockState) {
    var wrappedBlockKeys = getWrappedBlockKeys(wrappedBlockState.document.nodes);

    var state = wrappedBlockState;
    if (wrappedBlockKeys.size > 0) {
      var transform = state.transform();
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = wrappedBlockKeys[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var wrappedBlockKey = _step.value;

          transform = transform.moveTo({ anchorKey: wrappedBlockKey, anchorOffset: 0 }).unwrapBlock(opts.type);
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      state = transform.apply();
    }
    return state;
  }

  /**
   * Auto wrap.
   *
   * @param  {Function} getState
   * @param  {Function} onChange
   */

  function autoWrap(getState, onChange) {
    var state = getState();
    var selection = state.selection;


    var removedAllWrappedBlockState = removeAllWrappedBlock(state);

    var transform = removedAllWrappedBlockState.transform().moveTo(selection);

    var currentNode = state.blocks.first();
    if (opts.condition(currentNode)) {
      transform = transform.wrapBlock(opts.type);
    }

    onChange(transform.apply());
  }

  /**
   * Return the plugin.
   *
   * @type {Object}
   */

  return {
    onSelect: onSelect,
    onKeyDown: onKeyDown,
    schema: getSchema()
  };
}

/**
 * Export.
 *
 * @type {Function}
 */

exports.default = AutoWrapBlock;